@@ -40,7 +40,19 @@ import System.Process
 import Tools
 import UrlAnalyse
 
-runCompile :: String -> FullConfig -> ImperativeMonad ()
+-- | Parse MediaWiki or HTML source to a parse tree and store the resulting
+-- parse tree in an encoded file. This procedure is called by calling
+-- mediawiki2latex from mediawiki2latex itself in order to free memory when the
+-- subprocess terminates.
+runCompile ::
+  -- | The temporary directory where to load the input wiki source of HTML code
+  -- from.
+  String ->
+  -- | The configuration for this run of mediawiki2latex.
+  FullConfig ->
+  -- | The action parsing the input and storing the result in the temporary
+  -- directory.
+  ImperativeMonad ()
 runCompile dir cfg =
   do
     t <- liftIO $ Tools.readFile (dir </> "input")
@@ -53,7 +65,19 @@ runCompile dir cfg =
     cr <- return (ppt (not (latexTables cfg)) (parseit p t))
     liftIO $ B.writeFile (dir </> "output") (encode cr)
 
-runNewTree :: String -> ImperativeMonad ()
+-- | Add the labels for the chapter, section, subsection etc. references to the
+-- parse tree and store the result as well as the updated state containing the
+-- list of labels in the temporary directory. This procedure is called by
+-- calling mediawiki2latex from mediawiki2latex itself in order to save memory
+-- by using that the memory of each subprocess is freed upon its termination. It
+-- also reads the original parse tree as well as the state information from the
+-- temporary directory.
+runNewTree ::
+  -- | The name of the temporary directory.
+  String ->
+  -- | The action adding the labels to the parse tree and writing the result to
+  -- the temporary directory.
+  ImperativeMonad ()
 runNewTree dir =
   do
     t <- liftIO $ B.readFile (dir </> "output")
@@ -66,7 +90,19 @@ runNewTree dir =
     liftIO $ B.writeFile (dir </> "newtree") (encode ntr)
     liftIO $ B.writeFile (dir </> "us") (encode nus)
 
-runTreeToLaTeX :: String -> String -> ImperativeMonad ()
+-- | Write the parse tree as LaTeX code to a temporary directory. This procedure
+-- is called by calling mediawiki2latex from mediawiki2latex itself in order to
+-- save memory by using that the memory of each subprocess is freed upon its
+-- termination. It also reads the parse tree from the temporary directory as
+-- well as the initial state of the LaTeX renderer and write the updated state
+-- to the temporary directory as well.
+runTreeToLaTeX ::
+  -- | The directory where to read the initial state from.
+  String ->
+  -- | The directory where to read the parse tree from and write the LaTeX code
+  -- as well as the updated state information to.
+  String ->
+  ImperativeMonad ()
 runTreeToLaTeX instfn dir =
   do
     t <- liftIO $ B.readFile (dir </> "newtree")
@@ -81,7 +117,7 @@ runTreeToLaTeX instfn dir =
     liftIO $ B.writeFile (dir </> "st") (encode newst)
     liftIO $ B.writeFile (dir </> "inst") (encode newst)
 
--- | main function to compile mediawiki pages   |
+-- | main function to compile MediaWiki pages
 compile ::
   RunMode ->
   String ->
@@ -190,14 +226,14 @@ compile
                     _ -> []
               (runcheap (getStyles hhelp) b mylanguage mytitle pt (hostname . fullUrl $ st) templates tabs fformulas theRunMode latexTabels)
 
--- | pathname of the temporary directory of the compiler |
+-- | pathname of the temporary directory of the compiler
 dirpref :: [Char]
 dirpref = "../tmp/compiler/"
 
 -- | converts a wiki source document received from mediawiki when requesting it
 -- for Special:Export to a parse tree to be converted to LaTeX be treeToLaTeX3.
 -- It also signals to compiler.py that the source code was read using the
--- temporary compiler directory |
+-- temporary compiler directory
 shortparse :: String -> IO [Anything Char]
 shortparse x =
   do
@@ -393,6 +429,9 @@ run
       theNewSizes = concat (map sizeFun theSizes)
       sizeFun (t, k) = map (\(s, b) -> ([t, s], b)) k
 
+-- | Run the compile to compile MediaWiki code or HTML to LaTeX but save memory
+-- by running as many steps as possible in separate processes. This is because
+-- the memory of each subprocess is released once it terminates.
 runcheap ::
   [Anything Char] ->
   Bool ->
