@@ -27,7 +27,14 @@ import Tools
 import UrlAnalyse
 import WikiHelper
 
-modpath2 :: String -> URL -> URL
+-- | Get an URL to a media description page on a wiki.
+modpath2 ::
+  -- | The name of the media file
+  String ->
+  -- | An URL to a Wiki currently being processed
+  URL ->
+  -- | The URL to the media decription page of the file on the wiki
+  URL
 modpath2 s u =
   u
     { url_path =
@@ -39,7 +46,15 @@ modpath2 s u =
       ('/' : xs) -> (reverse xs)
       xs -> (reverse xs)
 
-conv :: URL -> String -> String
+-- | Convert an URL to the original media file read from the media description
+-- page of the media file to and ready to be passed to curl for download.
+conv ::
+  -- | A base URL see documentation on `WikiUrl` for details.
+  URL ->
+  -- | The URL read from the media description page.
+  String ->
+  -- | The URL ready to be used in curl for download.
+  String
 conv u s =
   if take 5 s == "http:"
     then s
@@ -67,15 +82,38 @@ conv u s =
                   "%"
               )
 
-getImageUrl2 :: (String, URL) -> Maybe String
+-- | Extract the URL to download a media file from the image description page of
+-- the media file on the wiki
+getImageUrl2 ::
+  -- | A tuple the first element is the HTML version of the image description
+  -- page of the media file on the wiki. The second element is a base URL to the
+  -- wiki currently being processed (see see documentation on `WikiUrl` for
+  -- details)
+  (String, URL) ->
+  -- | The URL to download the media file from the wiki server.
+  Maybe String
 getImageUrl2 (s, u) =
   (getImageUrl "fullImageLink" u s)
     `mplus` (getImageUrl "fullMedia" u s)
 
-getImageUrl3 :: String -> Maybe String
-getImageUrl3 s = return s
-
-getImageUrl :: String -> URL -> String -> Maybe String
+-- | Extract an URL from  to download a media file from the image description
+-- page of the media file on the wiki for on key string that is close to the
+-- place in the HTML page where the link is located. The method this function
+-- uses is also called HTML scraping. So it does not a full parse of HTML page
+-- since the page might not be valid UTF8. Instead it curs the HTML a certain
+-- marker strings in order to produce the result.
+getImageUrl ::
+  -- | The key string which is located closely to the download link in the HTML
+  -- page
+  String ->
+  -- | A base URL see documentation on `WikiUrl` for details.
+  URL ->
+  -- | The HTML version of the media description page downloaded from the wiki.
+  String ->
+  -- | The URL to download the media file from the MediaWiki server wrapped in a
+  -- Just value of the Maybe monad if found. A Nothing value of the Maybe monad
+  -- otherwise.
+  Maybe String
 getImageUrl fi u ss =
   if isInfixOf fil s
     then case splitOn fil s of
@@ -96,24 +134,27 @@ getImageUrl fi u ss =
     theHref = BStr.unpack (UTF8Str.fromString "href=\"")
     q = BStr.unpack (UTF8Str.fromString "\"")
 
--- | downloads a single image form the wiki.  It takes the temporary image
--- download directory as first parameter. It takes the WikiUrl of the wiki
--- website currently being processed as second parameter. It takes a tuple as
--- third input parameter. The first element of the tuple is the image number so
--- just an integer that can be used to identify the image uniquely) . The second
--- element of the tuple is image include string of the image from the wiki
--- source, that is the text in between the square brackets as second input
--- parameter. It returns a tuple. The first element of the tuple is a String
--- that shall be passed as part of a configuration file to the curl program in
--- order to download the all possible image description pages from the wiki. The
--- second element of the tuple is a list of output files where the curl program
--- shall store the image description page from the wiki on the local disk. The
--- third element of the tuple is a list of URLs where the image description
--- pages from the wiki are located on the web.
+-- | Function to calculate the download instruction for the curl program to
+-- download a single image
 getImagePage1 ::
+  -- | The temporary image download directory.
   String ->
+  -- | The WikiUrl of the wiki website currently being processed.
   WikiUrl ->
+  -- | A tuple. The first element of the tuple is the image number so just an
+  -- integer that can be used to identify the image uniquely). The second
+  -- element of the tuple is image include string of the image from the wiki
+  -- source, that is the text in between the square brackets but with the
+  -- "File:" or similar at the beginning removed and with everything after the |
+  -- removed. So just the plain image filename on the wiki.
   (Integer, String) ->
+  -- |  A tuple. The first element of the tuple is a String that shall be passed
+  -- as part of a configuration file to the curl program in order to download
+  -- all possible image description pages from the wiki.  The second element of
+  -- the tuple is a list of output files where the curl program shall store the
+  -- image description page from the wiki on the local disk. The third element
+  -- of the tuple is a list of URLs where the image description pages from the
+  -- wiki are located on the web.
   (String, [String], [String])
 getImagePage1 dir u (i, ss) =
   (concat (map go numberedurls), map moo numberedurls, myloadurls)
@@ -128,10 +169,23 @@ getImagePage1 dir u (i, ss) =
     kds (x : xs) = x : (kds xs)
     kds [] = []
 
+-- | Determine the link to download a media file from the wiki server.
 getImagePage2 ::
+  -- | The directiry where the HTML versions of the media desciption pages are
+  -- stored.
   String ->
+  -- | The `WikiUrl` to the wiki article currently being processed.
   WikiUrl ->
+  -- | A pair. Its first element is the image number. Its second element is the
+  -- image name.
   (Integer, String) ->
+  -- | A tuple wrapped into the a Just value of the Maybe monad if the URL to
+  -- download the media file could be determined. A Nothing value of the Maybe
+  -- monad otherwise. The first element of the tuple is a list of possible URLs
+  -- where the media file can be downloaded. The second element of the tuple is
+  -- the image number. The thrid element of the tuple is the URL to the media
+  -- decription page where the link to download the media file was found. The
+  -- fourth element of the tuple is the link to download the media file.
   IO (Maybe ([String], Integer, URL, String))
 getImagePage2 dir u (i, ss) =
   do
@@ -159,15 +213,14 @@ getImagePage2 dir u (i, ss) =
     go (uu, Just x) = [(uu, x)]
     go _ = []
 
--- | downloads a single image form the wiki.  It takes the temporary image
--- download directory as first parameter. It takes a tuple as second input
--- parameter. The first element of the tuple is the image number so just an
--- integer that can be used to identify the image uniquely) . The second element
--- of the tuple is image include string of the image from the wiki source, that
--- is the text in between the square brackets as second input parameter. It
--- takes the WikiUrl of the wiki website currently being processed as thrird
--- parameter. See function getImages in this module for documentation on the
--- returned data type
+-- | It takes the temporary image download directory as first parameter. It
+-- takes a tuple as second input parameter. The first element of the tuple is
+-- the image number so just an integer that can be used to identify the image
+-- uniquely) . The second element of the tuple is image include string of the
+-- image from the wiki source, that is the text in between the square brackets
+-- as second input parameter. It takes the WikiUrl of the wiki website currently
+-- being processed as third parameter. See function `getImages1` in this module
+-- for documentation on the returned data type
 doImage1 ::
   String -> WikiUrl -> (Integer, String) -> IO (String, [String], [String])
 doImage1 dir theWikiUrl img =
@@ -178,8 +231,21 @@ doImage1 dir theWikiUrl img =
         (_ : xs) -> replace2 xs "%27" "'"
         _ -> []
 
+-- | Determine the URL to download a media file from the wiki.
 doImage2 ::
-  String -> WikiUrl -> (Integer, String) -> IO (Maybe String)
+  -- | The directory where the HTML versions of the media description pages are
+  -- stored on the local disk.
+  String ->
+  -- | The `WikiUrl` to the article currently being processed
+  WikiUrl ->
+  -- | A pair. Its first element is the image number. Its second element is the
+  -- image inclusion string. The is what is written in between the double square
+  -- brackets in the wiki code.
+  (Integer, String) ->
+  -- | The URL to download the media file from the wiki wrapped into a Just
+  -- value of the Maybe monad if it could be determined. A Nothing value of the
+  -- Maybe monad otherwise.
+  IO (Maybe String)
 doImage2 dir theWikiUrl img =
   do
     p <- getImagePage2 dir theWikiUrl (fst img, theName)
@@ -192,9 +258,21 @@ doImage2 dir theWikiUrl img =
         (_ : xs) -> replace2 xs "%27" "'"
         _ -> []
 
+-- | Determine the author and license information for the media description page
+-- on the wiki and write it to the local disk. Also write the link to the
+-- version history page of the media description page to disk. This function
+-- makes a call from the mediawiki2latex executable to itself as a subprocess in
+-- order to save memory since the memory of a subprocess is free upon its
+-- termination.
 getImgContribUrl ::
+  -- | The host name of the MediaWiki server where the content is located
   String ->
+  -- | The file name on the local disk where the HTML version of the media
+  -- description page is located.
   String ->
+  -- | The action running the subprocess to determine the author and license
+  -- information of the media file as well as the link to the version history
+  -- page of the image description page on the wiki.
   IO ()
 getImgContribUrl theHost x =
   do
@@ -205,8 +283,14 @@ getImgContribUrl theHost x =
         )
     return ()
 
+-- | Call the mediawiki2latex executable by itself to determine the contributor
+-- information of a page on the wiki.
 getImgContrib ::
+  -- | The file where the version history page of the page on the wiki has been
+  -- stored on the local disk.
   String ->
+  -- | The IO action to determine the contributor information from the version
+  -- histroy page.
   IO ()
 getImgContrib theFileName =
   do
@@ -217,8 +301,16 @@ getImgContrib theFileName =
         )
     return ()
 
+-- | Determine the contributors of a page on the wiki from its version history
+-- page and write it to disk. This function is called by the mediawiki2latex
+-- executable calling itself in a new subprocess. This is done to save memory
+-- since the memory of a subprocess is freed upon its termination.
 getContribCallBack ::
-  [Char] -> ImperativeMonad ()
+  -- | The file where the version histroy page is stored on the local disk.
+  [Char] ->
+  -- | The action to parse the version histroy page and write the contributor
+  -- information to disk.
+  ImperativeMonad ()
 getContribCallBack theFileName =
   do
     x <- liftIO (Tools.readFile theFileName)
@@ -242,7 +334,20 @@ getContribCallBack theFileName =
     go _ = ("", "")
     go2 (c, (a, h)) = (a, h, c)
 
-runCtrbUrl :: String -> String -> ImperativeMonad ()
+-- | Determine the author(s) (contributor(s)) of a media file. This function is
+-- called by mediawiki2latex calling itself in a subprocess in order to save
+-- memory, since the memory of a subprocess is automatically freed upon its
+-- termination.
+runCtrbUrl ::
+  -- | The filename of the file where the HTML code of the media description
+  -- page from the wiki is stored on the local disk.
+  String ->
+  -- | The host name of the MediaWiki server where the content is located.
+  String ->
+  -- | The action to parse the HTML code, write the author and license
+  -- information as well as the URL to the version history page of the media
+  -- description page of the media file on the wiki to the local disk.
+  ImperativeMonad ()
 runCtrbUrl ctrbfilename theHost =
   do
     yy <- liftIO (Tools.readFile ctrbfilename)
@@ -284,22 +389,31 @@ runCtrbUrl ctrbfilename theHost =
     liftIO (Tools.writeFile (ctrbfilename ++ ".histurl") theUrl)
     return ()
 
--- | main function to download images. It takes the temporary image download
--- directory as first parameter. It takes image include strings of the images
--- from the wiki source, that is the text in between the square brackets as
--- second input parameter. It takes the WikiUrl of the wiki websitze currently
--- being processed as thrird parameter. This function runs as a background
--- process. So it returns a list of empty MVars immediately when being called.
--- They are later on filled with information on the downloaded images including
--- their location in the temporary image download directory. The returend MVars
--- contain ImageInfo values. See description in the module ImperativeState for a
--- detailed description.
-fullmake :: String -> String -> IO ()
+-- | Main function to download files with curl.
+fullmake ::
+  -- | The temporary image download directory.
+  String ->
+  -- | The number of the curl main run as string.
+  String ->
+  -- | The IO action downloading the files from the wiki server.
+  IO ()
 fullmake dir s = do
   _ <- system ("curl --parallel-max 2 --retry-all-errors --retry 3 -s -w \"%{http_code} %{url} %{filename_effective}\\n\" --compressed -K " ++ (dir </> ("curlrun" ++ s)) ++ " --parallel -L > " ++ (dir </> ("curloutput" ++ s ++ ".1")))
   mymake dir s 1
 
-mymake :: String -> String -> Integer -> IO ()
+-- | Sub function to download files with curl. This function tries to download
+-- the files as often as needed until all files were downloaded successfully.
+mymake ::
+  -- | The temporary image download directory.
+  String ->
+  -- | The number of the curl main run as string.
+  String ->
+  -- | The number of the curl sub run as string. If everything goes well only
+  -- one sub run is needed. But if some downloads fail as many sub runs as
+  -- needed will be done until all file were downloaded successfully.
+  Integer ->
+  -- | The IO action downloading the files from the wiki server.
+  IO ()
 mymake dir s n = do
   text <- Tools.readFile (dir </> ("curloutput" ++ s ++ "." ++ (show n)))
   let list = (filter ppred (map (splitOn " ") (splitOn "\n" text)))
@@ -314,8 +428,19 @@ mymake dir s n = do
     jjoin (_ : y : z : []) = "url = \"" ++ y ++ "\"\noutput = \"" ++ z ++ "\"\n"
     jjoin _ = []
 
+-- | Main function to download images from the wiki server. Also gets license
+-- and contributor information of the images.
 getImages1 ::
-  String -> [String] -> WikiUrl -> ImperativeMonad ([ImageCredits])
+  -- | The temporary image download directory.
+  String ->
+  -- | The names of the images. That is the list of image names. An image name
+  -- is what is in between the double square brackets in the wiki text.
+  [String] ->
+  -- | The `WikiUrl` to the main article currently being processed.
+  WikiUrl ->
+  -- | Action to download the images returning a list of `ImageCredits` records.
+  -- Each record represents the author and license information for an image.
+  ImperativeMonad ([ImageCredits])
 getImages1 dir imagess theWikiUrl =
   do
     liftIO $
