@@ -1,3 +1,4 @@
+-- | Module to convert HTML to a parse tree
 module HtmlParser where
 
 import qualified Data.Map as Map
@@ -5,14 +6,23 @@ import MediaWikiParseTree
 import MediaWikiParser
 import Text.HTML.TagSoup
 
+-- | Parse HTML with own HTML parser.
 parseHtml :: String -> [Anything Char]
 parseHtml = parseit minparsers
 
-data Frame = Frame {tag :: Tag String, ds :: [Anything Char]}
+-- | A stack frame for the and HTML tag with HTML element it contains
+data Frame = Frame
+  { -- | The HTML tag
+    tag :: Tag String,
+    -- | The HTML elements with a the content of the tag.
+    ds :: [Anything Char]
+  }
 
+-- | Parse HTML with tagsoup parser
 parseHtmlFast :: String -> [Anything Char]
 parseHtmlFast = makebrackets . parseTags
 
+-- | Convert the tag stream of the tagsoup parser to a parse tree
 makebrackets :: [Tag String] -> [Anything Char]
 makebrackets l = go l [Frame {tag = (TagOpen "root" []), ds = []}]
   where
