@@ -405,7 +405,11 @@ qBookIncludeAction cfg wurl text =
       (ExpandedTemplates _) -> (loadMediaWiki d wurl) >>= (return . Just)
       _ -> (loadPlain d wurl Nothing) >>= (return . Just)
 
-langau :: Map String String
+-- | Mapping from the language code (e.g. en for English) to the Heading used in
+-- the list of contributors (e.g. Contributors for English).
+langau ::
+  -- | Map from language code to the heading for the list of contributors.
+  Map String String
 langau =
   fromList
     [ ("hi", "\2354\2375\2326\2325"),
@@ -444,7 +448,16 @@ langau =
       ("br", "Aozer")
     ]
 
-makeHeader :: FullWikiUrl -> Maybe String -> [Char]
+-- | Returns the header for the list of contributors in LaTeX version.
+makeHeader ::
+  -- | The `FullWikiUrl` of the wiki page to be processed.
+  FullWikiUrl ->
+  -- | The language code (e.g. en for English) for the wiki to be processed
+  -- wrapped in a Just value of the Maybe monad if it could be determined. A
+  -- Nothing value of the Maybe monad otherwise.
+  Maybe String ->
+  -- | The header of the list of contributors in LaTeX version.
+  [Char]
 makeHeader fullurl m =
   let mmm = m >>= (\yy -> Map.lookup yy langau)
    in "\\chapter{"
@@ -462,7 +475,16 @@ makeHeader fullurl m =
         ++ "\\begin{longtable}{rp{0.6\\linewidth}}\n"
         ++ "\\textbf{Edits}&\\textbf{User}\\\\\n"
 
-makeHeaderHTML :: FullWikiUrl -> Maybe String -> [Char]
+-- | Returns the header for the list of contributors in HTML version.
+makeHeaderHTML ::
+  -- | The `FullWikiUrl` of the wiki page to be processed.
+  FullWikiUrl ->
+  -- | The language code (e.g. en for English) for the wiki to be processed
+  -- wrapped in a Just value of the Maybe monad if it could be determined. A
+  -- Nothing value of the Maybe monad otherwise.
+  Maybe String ->
+  -- | The header of the list of contributors in HTML version.
+  [Char]
 makeHeaderHTML fullurl m =
   let mmm = m >>= (\yy -> Map.lookup yy langau)
    in "<h2>"
@@ -479,7 +501,15 @@ makeHeaderHTML fullurl m =
         ++ "<table rules=\"all\">"
         ++ "<tr><td>Edits</td><td>User</td></tr>"
 
-makeBody :: (Ord t) => Map t Contributor -> URL -> [Char]
+-- | Returns the body of the list of contributors in LaTeX version.
+makeBody ::
+  -- | A mapping from the names of the contributor their respective contributor
+  -- records.
+  Map String Contributor ->
+  -- | The URL to the wiki page currently being processed.
+  URL ->
+  -- | The body of the list of contributors in LaTeX version.
+  [Char]
 makeBody m u = concat (map go (sort (toList m)))
   where
     fun ('/' : xs) = xs
@@ -494,7 +524,15 @@ makeBody m u = concat (map go (sort (toList m)))
         ++ (concat (map chartrans (name v)))
         ++ "}\\\\\n"
 
-makeBodyHTML :: (Ord t) => Map t Contributor -> URL -> [Char]
+-- | Returns the body of the list of contributors in HTML version.
+makeBodyHTML ::
+  -- | A mapping from the names of the contributor their respective contributor
+  -- records.
+  Map String Contributor ->
+  -- | The URL to the wiki page currently being processed.
+  URL ->
+  -- | The body of the list of contributors in HTML version.
+  [Char]
 makeBodyHTML m u = concat (map go (sort (toList m)))
   where
     fun ('/' : xs) = xs
@@ -508,7 +546,14 @@ makeBodyHTML m u = concat (map go (sort (toList m)))
         ++ (concat (map chartrans (name v)))
         ++ "</a></td></tr>"
 
-makeContributors :: Maybe URL -> ImperativeMonad (String, String)
+-- | Returns the list of contributors in LaTeX and HTML version.
+makeContributors ::
+  -- | The URL of the wiki page currently being processed.
+  Maybe URL ->
+  -- | Returns a tuple wrapped into the imperative monad. The first element of
+  -- the tuple is the LaTeX version of the list of contributors. The second
+  -- element of the tuple is the HTML version of the list of contributors.
+  ImperativeMonad (String, String)
 makeContributors uu =
   do
     st <- get
@@ -534,35 +579,34 @@ makeContributors uu =
           ++ "</table>"
       )
 
-parseUrl :: String -> ImperativeMonad FullWikiUrl
+-- | Parse an URL to a `FullWikiUrl`. Throw an error if the parsing was not
+-- successful, return the `FullWikiUrl` otherwise. See also documentation of the
+-- the function `analyseFull`.
+parseUrl ::
+  -- | The URL to be analyzed .
+  String ->
+  -- | The parsed version of the URL given in the first parameter.
+  ImperativeMonad FullWikiUrl
 parseUrl u =
   case analyseFull u of
     Just x -> return x
     _ -> throwError (WikiUrlParseError u)
 
-getContributors ::
-  [String] ->
-  ImperativeMonad (([(Map String Contributor)], [(Maybe String)]))
-getContributors u =
-  do
-    st <- get
-    stz <- liftIO imperativeStateZero
-    put stz
-    au <- mapM go u
-    newState <- get
-    put st
-    rr <- liftIO (return (audict newState, au))
-    return rr
-  where
-    go uu =
-      do
-        purl <- parseUrl uu
-        sst <- get
-        put sst {fullUrl = purl}
-        addContributors (lemma purl) (Just (UrlAnalyse.url purl))
-
+-- | Determine the contributors of an article on the wiki and update the state
+-- of ImperativeMonad according to this information. Return the license of the
+-- wiki page wrapped in a Just value of the Maybe monad if it could be
+-- determine. Return a Nothing value of the Maybe monad otherwise.
 addContributors ::
-  [Char] -> Maybe URL -> ImperativeMonad ((Maybe String))
+  -- | The name of the article of which to determine the authors.
+  [Char] ->
+  -- | The URL to the main page on the wiki to be processed.
+  Maybe URL ->
+  -- | The license of the wiki page wrapped into a Just value of the Maybe monad
+  -- if it could be determined. The Nothing value of the Maybe monad otherwise.
+  -- The whole returned value is wrapped into the ImperativeMonad. The state of
+  -- the ImperativeMonad is updated with new contributor information of the
+  -- article currently being processed.
+  ImperativeMonad ((Maybe String))
 addContributors theLemma uu =
   do
     sst <- get
@@ -616,7 +660,21 @@ noinclude ::
 noinclude wurl =
   runAction "<noinclude>" "</noinclude>" (zeroAction wurl)
 
-runActions :: WikiUrl -> String -> String -> ImperativeMonad String
+-- | Resolve the references to included sub pages when processing wiki text and
+-- removed content between noinclude tags.
+runActions ::
+  -- | The `WikiUrl` the main page to be processed.
+  WikiUrl ->
+  -- | The name of the main article to be processed.
+  String ->
+  -- | The wiki source text of the article before the resolution of possibly
+  -- included sub pages.
+  String ->
+  -- | The wiki source text with the references to sub pages resolved up to a
+  -- certain maximum recursion depth wrapped in to imperative monad. Also
+  -- updates the state of Imperative monad with the contributor information
+  -- determined for each sub page.
+  ImperativeMonad String
 runActions wurl lema text =
   do
     x <- noinclude wurl text
@@ -656,26 +714,21 @@ runBookActions fu text cfg =
   where
     wurl = wikiUrl fu
 
-replacements :: String -> String
-replacements x =
-  replace2
-    ( replace2
-        ( replace2
-            ( replace2
-                x
-                "[[Image:Nuvola apps noatun.png|left|20px|Aufgabe]]"
-                "{{Ubung}}"
-            )
-            "[[Image:Yes_check.svg|12px]]"
-            "{{TickYes}}"
-        )
-        "{{col-break}}"
-        "\n|"
-    )
-    "{{Fortran:Vorlage: Table}}"
-    "prettytable"
-
-loadPlain :: String -> WikiUrl -> Maybe URL -> ImperativeMonad [Char]
+-- | Loads the wiki source code of a wiki page and resolve possibly included sub
+-- pages. Also determines the contributors of the wiki page and any possibly
+-- included sub pages.
+loadPlain ::
+  -- | The name of the article on the wiki .
+  String ->
+  -- | The `WikiUrl` to the article to be processed.
+  WikiUrl ->
+  -- | The URL to the article to be processed wrapped into a Just value of the
+  -- Maybe monad if known. Nothing value of the Maybe monad otherwise
+  Maybe URL ->
+  -- | The wiki source of the page with sub page inclusions expanded wrapped
+  -- into the ImperativeMonad. Also adds the contributor information of the wiki
+  -- page and all included sub pages to the state of the ImperativeMonad.
+  ImperativeMonad [Char]
 loadPlain lema wurl uu =
   do
     pp <- liftIO (getpage lema wurl)
@@ -711,7 +764,17 @@ loadBook st uu cfg =
             runBookActions fu p cfg
           _ -> throwError (DownloadError (lemma fu) (exportURL (url fu)))
 
-loadHTML :: ImperativeState -> ImperativeMonad String
+-- | Load an HTML page from the wiki. Also gets the contributor information of
+-- that page from the wiki. See `loadPlain` for details of the contributor
+-- determination process.
+loadHTML ::
+  -- | The state of the ImperativeMonad at the time when this function is
+  -- called.
+  ImperativeState ->
+  -- | The HTML page as a String wrapped into the ImperativeMonad. Also the
+  -- state of the ImperativeMonad is updated to account for the additionally
+  -- determined contributors.
+  ImperativeMonad String
 loadHTML st =
   let fu = fullUrl st
    in do
