@@ -61,8 +61,6 @@ data EnvType
     TemplateInside
   | -- | The contents inside a template whose inner structure shall not be parsed. Needed for templates containing source code, which shall be passed to latex verbatim.
     TemplateInsideVerbatim
-  | -- | A chapter heading. In wiki notation this is denoted by single equal signs surrounding the chapter heading.
-    Chapter
   | -- | A gallery. So a group of images that belong together.
     Gallery
   | -- | content inside a nowiki tag in wiki code.
