@@ -23,24 +23,13 @@ import Text.ParserCombinators.Parsec.Prim
 import Tools
 import UrlAnalyse
 
-makeUrl2 :: String -> String -> [Char]
-makeUrl2 theLemma theHost =
-  (unify . exportURL)
-    ( URL
-        { url_path = "w/index.php",
-          url_params =
-            [ ("title", (replace2 (replace2 theLemma "%27" "'") "%" "%25")),
-              ("offset", ""),
-              ("limit", "500000"),
-              ("action", "history")
-            ],
-          url_type =
-            Absolute
-              (Host {protocol = HTTP True, host = theHost, port = Nothing})
-        }
-    )
-
-makeUrl4 :: String -> [Char]
+-- | Generate an URL linking to the version history of a page form the URL to
+-- the page itself.
+makeUrl4 ::
+  -- | The URL to the page.
+  String ->
+  -- | The URL to the version history of the page given in the first parameter.
+  [Char]
 makeUrl4 uuu =
   fromMaybe
     uuu
@@ -62,7 +51,15 @@ makeUrl4 uuu =
             )
     )
 
-makeUrl3 :: String -> String -> [Char]
+-- | Generate an URL from a lemma on a wiki (e.g. an name of an article on
+-- Wikipedia) and a host name where the server of the wiki resides.
+makeUrl3 ::
+  -- | The lemma on the wiki
+  String ->
+  -- | The host name of the server serving the wiki
+  String ->
+  -- | The URL linking to the lemma on the server.
+  [Char]
 makeUrl3 theLemma theHost =
   (unify . exportURL)
     ( URL
@@ -74,7 +71,15 @@ makeUrl3 theLemma theHost =
         }
     )
 
-deepGet2 :: [Char] -> [Anything a] -> [Anything a]
+-- | Get all tag with a given tag name from a parse tree as a list of parse tree
+-- elements.
+deepGet2 ::
+  -- | The name of the tag to be looked for.
+  [Char] ->
+  -- | The parse tree to be investigated.
+  [Anything a] ->
+  -- | The list of parse tree element with the given tag name.
+  [Anything a]
 deepGet2 tag ll = concat $ map go ll
   where
     go (Environment Tag (TagAttr t m) l)
@@ -83,7 +88,14 @@ deepGet2 tag ll = concat $ map go ll
     go (Environment _ _ l) = (deepGet2 tag l)
     go _ = []
 
-getLicense :: [Anything Char] -> Maybe [Char]
+-- | Get the license of a media file from the parse tree of the media
+-- description page e.g. an image description page on wikimedia commons.
+getLicense ::
+  -- | The parse tree representing the media description page .
+  [Anything Char] ->
+  -- | A Just value of the maybe monad containing the acronym of the licenses
+  -- under which the media file is licensed.
+  Maybe [Char]
 getLicense l = (go (killTree l))
   where
     go :: [Anything Char] -> Maybe String
@@ -99,7 +111,15 @@ getLicense l = (go (killTree l))
       [Environment x y (killTree ll)]
     killNode x = [x]
 
-getAuthor :: [Anything Char] -> Maybe [Anything Char]
+-- | Get an author as a parse tree section from the information template on a
+-- media file, e.g. an image on wikimedia commons.
+getAuthor ::
+  -- | The parse tree to be inspected. E.g. as downloaded and parsed from an
+  -- image description page on wikimedia commons.
+  [Anything Char] ->
+  -- | The section of the parse tree containing the information on the author of
+  -- the media file
+  Maybe [Anything Char]
 getAuthor x = listToMaybe (concat (map go (deepGet2 "tr" x)))
   where
     go (Environment _ _ l) =
@@ -111,11 +131,26 @@ getAuthor x = listToMaybe (concat (map go (deepGet2 "tr" x)))
             _ -> []
     go _ = []
 
+-- | Find out the contributors of a media file.
 simpleContributors ::
+  -- | The lemma (name of the article) under which the media file of which the
+  -- contributors shall be determined is stored on the wiki.
   [Char] ->
+  -- | The host name of the wiki server where the article is stored
   [Char] ->
+  -- | The URL under which the media file of which the contributors shall be
+  -- determined is stored.
   Maybe URL ->
+  -- | The state of the part of the program with imperative flow of control (see
+  -- `ImperativeState.ImperativeState`)
   ImperativeState ->
+  -- | A list of tuples. The first element of each tuple is the name of a
+  -- contributor in LaTeX notation. The second element of the tuple is a link to
+  -- the wiki page of the contributor. The third element of each tuple is the
+  -- number of contributions of the given contributor to the given wiki page.
+  -- The fourth element of each tuple is the abbreviation of the license of the
+  -- media file as a Just value of the Maybe monad if it could be determined and
+  -- a Nothing value of the Maybe monad otherwise.
   IO [(String, String, Int, Maybe String)]
 simpleContributors theLemma theHost uu st =
   do
@@ -207,15 +242,35 @@ simpleContributors theLemma theHost uu st =
           _ -> return []
         return ((map go2 (zip (map (count ll) n) n)) ++ next)
 
-count :: (Eq a) => [a] -> a -> Int
+-- | Count the number of occurrences of the item given as second parameter in
+-- the list given as first parameter.
+count ::
+  (Eq a) =>
+  -- | The list in which the occurrences of the item given in the second
+  -- parameter shall be counted.
+  [a] ->
+  -- | The item the occurrences of which in the list given in the first
+  -- parameter shall be counted.
+  a ->
+  -- | The number of occurrences in the list given as first of the item given in
+  -- the second parameter
+  Int
 count l s = length (filter (== s) l)
 
-pre :: (String, String) -> Bool
+-- | A predicate returning True if the first element of the pair given as input
+-- parameter is an not IP address.
+pre ::
+  -- | A pair. The first element is checked for being an IP address.
+  (String, String) ->
+  -- | True if the first element of the pair given as first parameter is an not
+  -- IP address.
+  Bool
 pre s =
   case (runParser ipaddr () "" (fst s)) of
     Right _ -> False
     Left _ -> True
 
+-- | A parser for a single integer digit.
 intdigit :: Parser Int
 intdigit =
   do
@@ -224,6 +279,7 @@ intdigit =
       [(i, [])] -> return i
       _ -> pzero
 
+-- | A parser for a part of an IPv4 address, which is a three digits.
 ipnum3 :: ParsecT String () Identity Int
 ipnum3 =
   do
@@ -232,6 +288,7 @@ ipnum3 =
     c <- intdigit
     return (a * 100 + b * 10 + c)
 
+-- | A parser for a part of an IPv4 address, which is two digits.
 ipnum2 :: ParsecT String () Identity Int
 ipnum2 =
   do
@@ -239,15 +296,18 @@ ipnum2 =
     b <- intdigit
     return (a * 10 + b)
 
+-- | A parser for a part of an IPv4 address, which is a single digit.
 ipnum1 :: Parser Int
 ipnum1 = do intdigit
 
+-- | A parser for a part of an IPv4 address
 ipnum :: ParsecT [Char] () Identity ()
 ipnum =
   do
     n <- (try (ipnum3)) <|> (try (ipnum2)) <|> ipnum1
     if ((n <= 255) && (n >= 0)) then return () else pzero
 
+-- | A parser for an IPv4 address or an IPv6 address.
 ipaddr ::
   Text.Parsec.Prim.ParsecT
     [Char]
@@ -256,6 +316,7 @@ ipaddr ::
     ()
 ipaddr = try (ipv4addr) <|> ipv6addr
 
+-- | A parser for an IPv4 address.
 ipv4addr ::
   Text.Parsec.Prim.ParsecT
     [Char]
@@ -273,6 +334,7 @@ ipv4addr =
     _ <- ipnum
     return ()
 
+-- | A parser for a four digit part of and IPv6 address.
 ipv6num ::
   Text.Parsec.Prim.ParsecT
     [Char]
@@ -288,6 +350,7 @@ ipv6num =
       _ <- try (hexDigit) <|> return '0'
       return ()
 
+-- | A parser for an IPv6 address.
 ipv6addr ::
   Text.Parsec.Prim.ParsecT
     [Char]
